/*_############################################################################
  _## 
  _##  SNMP4J-AgentJMX - MBeanMultiAttributeMOInfo.java  
  _## 
  _##  Copyright (C) 2006-2009  Frank Fock (SNMP4J.org)
  _##  
  _##  This program is free software; you can redistribute it and/or modify
  _##  it under the terms of the GNU General Public License version 2 as 
  _##  published by the Free Software Foundation.
  _##
  _##  This program is distributed in the hope that it will be useful,
  _##  but WITHOUT ANY WARRANTY; without even the implied warranty of
  _##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  _##  GNU General Public License for more details.
  _##
  _##  You should have received a copy of the GNU General Public License
  _##  along with this program; if not, write to the Free Software
  _##  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
  _##  MA  02110-1301  USA
  _##  
  _##########################################################################*/

package org.snmp4j.agent.mo.jmx;

import javax.management.*;
import java.io.IOException;
import org.snmp4j.agent.mo.jmx.types.*;

/**
 * There are cases where a single SNMP variable is mapped to several
 * MBean attributes. <code>MBeanMultiAttributeMOInfo</code> provides means
 * for such a mapping.
 *
 * @author Frank Fock
 * @version 1.0
 */
public class MBeanMultiAttributeMOInfo extends MBeanAttributeMOInfo {

  public MBeanMultiAttributeMOInfo(ObjectName name,
                                   CombinedTypedAttribute attributes) {
    super(name, attributes);
  }

  public CombinedTypedAttribute getAttributes() {
    return (CombinedTypedAttribute) super.getAttribute();
  }

  public Object getAttribute(MBeanServerConnection server) throws ReflectionException,
      InstanceNotFoundException, AttributeNotFoundException, MBeanException,
      IOException {
    return MBeanMultiAttributeMOInfo.getAttribute(server, getObjectName(),
                                                  getAttributes());
  }

  public static Object getAttribute(MBeanServerConnection server,
                                    ObjectName name,
                                    CombinedTypedAttribute attributes) throws
      ReflectionException,
      InstanceNotFoundException, AttributeNotFoundException, MBeanException,
      IOException
  {
    TypedAttribute[] attr = attributes.getAttributes();
    Object[] values = new Object[attr.length];
    for (int i=0; i<values.length; i++) {
      values[i] = server.getAttribute(name, attr[i].getName());
      values[i] = attr[i].transformFromNative(values[i], name);
    }
    return attributes.transformFromNative(values, name);
  }


  public void setAttribute(MBeanServerConnection server, Object value) throws
      ReflectionException, MBeanException, InvalidAttributeValueException,
      AttributeNotFoundException, InstanceNotFoundException, IOException {
    MBeanMultiAttributeMOInfo.setAttribute(server, getObjectName(),
                                           getAttributes(), value);
  }

  public static void setAttribute(MBeanServerConnection server,
                                  ObjectName name,
                                  CombinedTypedAttribute attributes,
                                  Object value) throws
      ReflectionException, MBeanException, InvalidAttributeValueException,
      AttributeNotFoundException, InstanceNotFoundException, IOException
  {
    TypedAttribute[] attr = attributes.getAttributes();
    Object[] values = new Object[attr.length];
    Object[] nativeValue;
    if (attributes.isNativeValueAlwaysNeeded()) {
      for (int i=0; i<values.length; i++) {
        values[i] = server.getAttribute(name, attr[i].getName());
      }
      nativeValue = (Object[]) attributes.transformToNative(value, values, null);
    }
    else {
      nativeValue = (Object[]) attributes.transformToNative(value, null, null);
    }
    for (int i=0; (i<nativeValue.length) && (i<attr.length); i++) {
      Object v = attr[i].transformToNative(nativeValue[i], values[i], null);
      if (v != null) {
        server.setAttribute(name, new Attribute(attr[i].getName(), v));
      }
    }
  }
}
